--
--      Name: Am2901 Register Arithmetic Logic Unit (am2901.vhd)
--
--      Description: This design implements a variable size Am2901
--      register ALU for use in microprogram designs.
--
--      Designer: Thomas Mitchell
--
--      Revision History: 21 November 2009 - Initial version (TDM)
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.Am2901_comp.all;

entity Am2901 is

  generic (
    SLICE_WIDTH     :     integer := 4);
  port (
    CP_in           : in  std_logic;
    A_in            : in  std_logic_vector(3 downto 0);
    B_in            : in  std_logic_vector(3 downto 0);
    D_in            : in  std_logic_vector((SLICE_WIDTH - 1) downto 0);
    I_in            : in  std_logic_vector(8 downto 0);
    nOE_in          : in  std_logic;
    CI_in           : in  std_logic;
    Q_LSB_in        : in  std_logic;
    Q_MSB_in        : in  std_logic;
    RAM_LSB_in      : in  std_logic;
    RAM_MSB_in      : in  std_logic;
    Y_out           : out std_logic_vector((SLICE_WIDTH - 1) downto 0);
    nG_out          : out std_logic;
    nP_out          : out std_logic;
    Feq0_out        : out std_logic;
    F_MSB_out       : out std_logic;
    OVR_out         : out std_logic;
    CO_out          : out std_logic;
    Q_LSB_out       : out std_logic;
    Q_MSB_out       : out std_logic;
    Q_LSB_Dir_out   : out std_logic;
    Q_MSB_Dir_out   : out std_logic;
    RAM_LSB_out     : out std_logic;
    RAM_MSB_out     : out std_logic;
    RAM_LSB_Dir_out : out std_logic;
    RAM_MSB_Dir_out : out std_logic);

end Am2901;

architecture arch_Am2901 of Am2901 is

  signal CP          : std_logic;
  signal A           : std_logic_vector(3 downto 0);
  signal B           : std_logic_vector(3 downto 0);
  signal D           : std_logic_vector((SLICE_WIDTH - 1) downto 0);
  signal I           : std_logic_vector(8 downto 0);
  signal OE          : std_logic;
  signal CI          : std_logic;
  signal Y           : std_logic_vector((SLICE_WIDTH - 1) downto 0);
  signal G           : std_logic;
  signal P           : std_logic;
  signal Feq0        : std_logic;
  signal F_MSB       : std_logic;
  signal OVR         : std_logic;
  signal CO          : std_logic;
  signal Q_LSB_Dir   : std_logic;
  signal Q_MSB_Dir   : std_logic;
  signal RAM_LSB_Dir : std_logic;
  signal RAM_MSB_Dir : std_logic;

  signal RAM_Shift_Out     : std_logic_vector((SLICE_WIDTH -1) downto 0);
  signal RAM_Shift_Cntrl   : std_logic_vector(2 downto 0);
  signal RAM_WE            : std_logic;
-- signal A_Data : std_logic_vector((SLICE_WIDTH - 1) downto 0);
-- signal B_Data : std_logic_vector((SLICE_WIDTH - 1) downto 0);
  signal Q                 : std_logic_vector((SLICE_WIDTH - 1) downto 0);
  signal R                 : std_logic_vector((SLICE_WIDTH - 1) downto 0);
  signal S                 : std_logic_vector((SLICE_WIDTH - 1) downto 0);
  signal MUX_A             : std_logic_vector((SLICE_WIDTH - 1) downto 0);
  signal MUX_B             : std_logic_vector((SLICE_WIDTH - 1) downto 0);
  signal ALU_Source_Select : std_logic_vector(2 downto 0);
  signal ALU_Cntrl         : std_logic_vector(2 downto 0);
  signal F                 : std_logic_vector((SLICE_WIDTH - 1) downto 0);
  signal Q_Shift_Out       : std_logic_vector((SLICE_WIDTH - 1) downto 0);
  signal Q_Shift_Cntrl     : std_logic_vector(2 downto 0);
  signal Output_Select     : std_logic_vector(2 downto 0);

begin  -- arch_Am2901

  CP <= CP_in;
  A  <= A_in;
  B  <= B_in;
  D  <= D_in;
  I  <= I_in;
  OE <= not nOE_in;
  CI <= CI_in;

  Y_out           <= Y;
  nG_out          <= not G;
  nP_out          <= not P;
  Feq0_out        <= Feq0;
  F_MSB_out       <= F_MSB;
  OVR_out         <= OVR;
  CO_out          <= CO;
  Q_LSB_Dir_out   <= Q_LSB_Dir;
  Q_MSB_Dir_out   <= Q_MSB_Dir;
  RAM_LSB_Dir_out <= RAM_LSB_Dir;
  RAM_MSB_Dir_out <= RAM_MSB_Dir;

  U1 : ram_shift
    generic map (
      WIDTH              => SLICE_WIDTH)
    port map (
      RAM_Shift_in       => F,
      RAM_Shift_Cntrl_in => RAM_Shift_Cntrl,
      RAM_LSB_in         => RAM_LSB_in,
      RAM_MSB_in         => RAM_MSB_in,
      RAM_LSB_out        => RAM_LSB_out,
      RAM_MSB_out        => RAM_MSB_out,
      RAM_LSB_Dir_out    => RAM_LSB_Dir,
      RAM_MSB_Dir_out    => RAM_MSB_Dir,
      RAM_Shift_Out      => RAM_Shift_Out);

  U2 : reg_ram
    generic map (
      WIDTH        => SLICE_WIDTH)
    port map (
      Clock_in     => CP,
      Data_in      => RAM_Shift_Out,
      A_Address_in => A,
      B_Address_in => B,
      RAM_WE_in    => RAM_WE,
      A_Data_out   => R,
      B_Data_out   => S);

-- A_Data_out => A_Data,
-- B_Data_out => B_Data);

-- R_Latch : process (CP)
-- begin                                -- process R_Latch
--     if CP'event and CP = '1' then    -- rising clock edge

-- R <= A_Data;

-- end if;
-- end process R_Latch;

-- S_Latch : process (CP)
-- begin                                -- process S_Latch
--     if CP'event and CP = '1' then    -- rising clock edge

-- S <= B_Data;

-- end if;
-- end process S_Latch;

  U3 : alu_source_mux
    generic map (
      WIDTH                => SLICE_WIDTH)
    port map (
      Direct_Data_in       => D,
      R_in                 => R,
      S_in                 => S,
      Q_Reg_in             => Q,
      ALU_Source_Select_in => ALU_Source_Select,
      MUX_A_out            => MUX_A,
      MUX_B_out            => MUX_B);

  U4 : q_shift
    generic map (
      WIDTH            => SLICE_WIDTH)
    port map (
      Q_Shift_in       => Q,
      F_in             => F,
      Q_Shift_Cntrl_in => Q_Shift_Cntrl,
      Q_LSB_in         => Q_LSB_in,
      Q_MSB_in         => Q_MSB_in,
      Q_LSB_out        => Q_LSB_out,
      Q_MSB_out        => Q_MSB_out,
      Q_LSB_Dir_out    => Q_LSB_Dir,
      Q_MSB_Dir_out    => Q_MSB_Dir,
      Q_Shift_out      => Q_Shift_Out);

  U5 : q_register
    generic map (
      WIDTH      => SLICE_WIDTH)
    port map (
      Clock_in   => CP,
      Q_Shift_in => Q_Shift_out,
      Q_out      => Q);

  U6 : alu
    generic map (
      WIDTH        => SLICE_WIDTH)
    port map (
      R_in         => MUX_A,
      S_in         => MUX_B,
      CI_in        => CI,
      ALU_Cntrl_in => ALU_Cntrl,
      F_out        => F,
      G_out        => G,
      P_out        => P,
      CO_out       => CO,
      F_MSB_out    => F_MSB,
      OVR_out      => OVR,
      Feq0_out     => Feq0);

  U7 : output_mux
    generic map (
      WIDTH            => SLICE_WIDTH)
    port map (
      A_Data_in        => R,
      F_in             => F,
      Output_Select_in => Output_Select,
      OE_in            => OE,
      Y_out            => Y);

  U8 : instruction_decode
    port map (
      I_in                  => I,
      RAM_WE_out            => RAM_WE,
      RAM_Shift_Cntrl_out   => RAM_Shift_Cntrl,
      Q_Shift_Cntrl_out     => Q_Shift_Cntrl,
      ALU_Cntrl_out         => ALU_Cntrl,
      ALU_Source_Select_out => ALU_Source_Select,
      Output_Select_out     => Output_Select);

end arch_Am2901;
